#!/usr/bin/perl -w
use strict;
use Test;
BEGIN { plan tests => 0 }
use Lingua::Slavic::Numbers qw( LANG_BG );

use vars qw(%numbers);

# DISABLED for now

# do 't/decimals';
# do 't/rig.pm';

# rig($numbers{LANG_BG()}, \&number_to_bg);
